#!/usr/bin/env python
#coding: utf-8
"""
Requirements
    - Python: http://www.python.org/
    - cherrypy: http://cherrypy.org/
    - cheetah: http://www.cheetahtemplate.org/
"""

import os
import sys
import cherrypy
import json
import sqlite3
import sqlite3

import sqlite3


from Cheetah.Template import Template


THISDIR = APPDIR = os.path.dirname(os.path.abspath(__file__))
INI_FILENAME = os.path.join(THISDIR, "ajaxtest_server.ini")
SQLITE_FILENAME = os.path.join(THISDIR, "mysqlitedb.sdb3")


def get_new_connection():
    """
    Gibt eine Verbindung zur SQLite-Datenbank zurück
    """
    
    return sqlite3.connect(SQLITE_FILENAME)


def get_threadsafe_connection():
    """
    Gibt jedem Thread seine eigene Datenbankverbindung.
    
    ACHTUNG! Das ist keine ausgeklügelte Funktion. Es wird nicht getestet,
    ob die Verbindung auch wirklich funktioniert. Dafür bin ich jetzt zu faul.
    
    ACHTUNG! Eine Connection die mit dieser Funktion geholt wurde, darf nicht 
    geschlossen werden.
    """
    
    thread_data = cherrypy.thread_data
    
    if not hasattr(thread_data, "connection"):
        thread_data.connection = get_new_connection()
    
    return thread_data.connection
    

def fill_testdatabase():
    """
    Eine Tabelle erstellen und ein paar Daten hinzufügen.
    """
    
    conn = get_new_connection()
    
    # Tabelle und Constraint erstellen
    cur = conn.cursor()
    sql = """
    CREATE TABLE IF NOT EXISTS addresses (
        id INTEGER PRIMARY KEY NOT NULL,
        first_name TEXT,
        last_name TEXT NOT NULL,
        CONSTRAINT addresses_unique UNIQUE (first_name, last_name)
    );
    """
    cur.execute(sql)
    cur.close()
    conn.commit()
    
    # Ein paar Einträge in die Tabelle schreiben
    addresses = [
        {"first_name": u"Max", "last_name": u"Mustermann"},
        {"first_name": u"Andrea", "last_name": u"Musterfrau"},
        {"first_name": u"Umlaute", "last_name": u"ÖöÄäÜü"},
    ]
    cur = conn.cursor()
    sql = """
    INSERT OR IGNORE INTO addresses (
        first_name, last_name
    ) VALUES (
        :first_name, :last_name
    );
    """
    cur.executemany(sql, addresses)
    conn.commit()
    
    # Fertig
    conn.close()


def get_all_addresses():
    """
    Liest die DB aus und gibt alle Adressen in einer Liste aus Dictionaries 
    zurück.
    """
    
    conn = get_threadsafe_connection()
    
    cur = conn.cursor()
    sql = """
    SELECT
        first_name,
        last_name
    FROM
        addresses
    ORDER BY
        first_name,
        last_name
    ;
    """
    cur.execute(sql)
    retlist = []
    for first_name, last_name in cur:
        retlist.append({
            "first_name": first_name,
            "last_name": last_name
        })
    cur.close()
    return retlist
    

class Root(object):
    
    
    def index(self, *args, **kwargs):
        template = Template(file = os.path.join(THISDIR, "index.tmpl"))
        return unicode(template)
    index.exposed = True


    def get_data(self, *args, **kwargs):
        addresses_list = get_all_addresses()
        json_string = json.dumps(addresses_list, indent = 2)
        cherrypy.response.headers["Content-Type"] = "text/x-json"
        return json_string
    get_data.exposed = True


def main():
    
    fill_testdatabase()
    
    app = cherrypy.tree.mount(Root(), config = INI_FILENAME)
    cherrypy.config.update({"tools.staticdir.root": APPDIR})
    cherrypy.quickstart(app, config = INI_FILENAME)


if __name__ == "__main__":
    main()


